import { Span } from '@opentelemetry/api';
import { InstrumentationConfig } from '@opentelemetry/instrumentation';
import type * as io from 'socket.io';
import type { HttpInstrumentation } from '@opentelemetry/instrumentation-http';
export declare const defaultSocketIoPath = "/socket.io/";
export declare const SocketIoInstrumentationAttributes: {
    SOCKET_IO_ROOMS: string;
    SOCKET_IO_NAMESPACE: string;
    SOCKET_IO_EVENT_NAME: string;
};
export interface SocketIoHookInfo {
    moduleVersion?: string;
    payload: any[];
}
export interface SocketIoHookFunction {
    (span: Span, hookInfo: SocketIoHookInfo): void;
}
export interface HttpTransportInstrumentationConfig {
    /** Set to the instance of `HttpInstrumentation` used for http instrumentation */
    httpInstrumentation: HttpInstrumentation;
    /** Set to the path of socket.io endpoint Desalts to `/socket.io/` */
    socketPath?: string;
}
export interface SocketIoInstrumentationConfig extends InstrumentationConfig {
    /** Hook for adding custom attributes before socket.io emits the event */
    emitHook?: SocketIoHookFunction;
    /** list of events to ignore tracing on for socket.io emits */
    emitIgnoreEventList?: string[];
    /** Hook for adding custom attributes before the event listener (callback) is invoked */
    onHook?: SocketIoHookFunction;
    /** list of events to ignore tracing on for socket.io listeners */
    onIgnoreEventList?: string[];
    /** Set to `true` if you want to trace socket.io reserved events (see https://socket.io/docs/v4/emit-cheatsheet/#Reserved-events) */
    traceReserved?: boolean;
    /** Set to `TransportInstrumentationConfig` if you want to filter out socket.io HTTP transport  */
    filterHttpTransport?: HttpTransportInstrumentationConfig;
}
export declare type Io = typeof io;
//# sourceMappingURL=types.d.ts.map